/*
 * Copyright 2008 Adam Tacy <adam.tacy AT gmail.com>
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.adamtacy.client.ui.effects;

import java.util.Iterator;
import java.util.Vector;

import org.adamtacy.client.ui.NEffectPanel;
import org.adamtacy.client.ui.effects.events.EffectCompletedEvent;
import org.adamtacy.client.ui.effects.events.EffectCompletedHandler;
import org.adamtacy.client.ui.effects.events.EffectInterruptedEvent;
import org.adamtacy.client.ui.effects.events.EffectInterruptedHandler;
import org.adamtacy.client.ui.effects.events.EffectStartingEvent;
import org.adamtacy.client.ui.effects.events.EffectStartingHandler;
import org.adamtacy.client.ui.effects.events.EffectSteppingEvent;
import org.adamtacy.client.ui.effects.events.EffectSteppingHandler;
import org.adamtacy.client.ui.effects.events.HasAllEffectEventHandlers;
import org.adamtacy.client.ui.effects.impl.css.StyleImplementation;
import org.adamtacy.client.ui.effects.transitionsphysics.EaseInOutTransitionPhysics;
import org.adamtacy.client.ui.effects.transitionsphysics.TransitionPhysics;

import com.google.gwt.animation.client.Animation;
import com.google.gwt.dom.client.Element;
import com.google.gwt.event.shared.EventHandler;
import com.google.gwt.event.shared.GwtEvent;
import com.google.gwt.event.shared.HandlerManager;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.ui.DockPanel;
import com.google.gwt.user.client.ui.HTMLPanel;
import com.google.gwt.user.client.ui.HTMLTable;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Panel;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.SimplePanel;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.user.client.ui.DockPanel.DockLayoutConstant;

/**
 * A class that provides the basics of an effect.
 * 
 * From v4 a new style of setting properties is introduced.
 * 
 * From v3 of GWT-FX this class is based on the GWT Animation class to keep
 * divergence from GWT itself as minimal as possible. To maintain the previous
 * API, this class extends Animation with a number of fields and methods.
 * 
 * It is an abstract class that provides a fair amount of functionality, but
 * leaves the implementation of the
 * 
 * @author adam
 * 
 */
public abstract class NEffect extends Animation implements
    HasAllEffectEventHandlers {

  public NEffectPanel getEffectPanel() {
    return thePanel;
  }

  /**
   * Inner class containing methods that are used to set properties of this base
   * effect class.
   * 
   * It is expected that this class is extended by Effects that wish to provide
   * additional user definable properties.
   * 
   * @deprecated Use new Parameter approach
   */
  protected class BaseProperties {
    /**
     * Return the effect length in seconds.
     * 
     * @return Effect length in seconds.
     * @deprecated Use getDuration() method in main class
     */
    public double getEffectLength() {
      return effectLengthSeconds;
    }

    /**
     * Return the current Transition Physics Model used by the Effect.
     * 
     * @return
     * @deprecated Use method in main class
     */
    public TransitionPhysics getTransitionPhysics() {
      return transition;
    }

    /**
     * Returns a boolean value indicating if the effect has finished or not.
     * 
     * @return True if the effect is finished; False otherwise
     * @deprecated Use method in main class
     */
    public boolean isFinished() {
      return effectFinished;
    }

    /**
     * Returns a boolean value to indicate if the effect is interruptable or
     * not.
     * 
     * @return True if the effect is interruptable; false otherwise.
     * @deprecated Use method in main class
     */
    public boolean isInterruptable() {
      return interruptable;
    }

    /**
     * Returns a boolean value to indicate if the effect is inverted.
     * 
     * @return
     * @deprecated Use method in main class
     */

    public boolean isInverted() {
      return inverted;
    }

    /**
     * Set the length of the effect.
     * 
     * @param new_EffectLengthSeconds The new double value (in seconds) for the
     *          effect length.
     * @deprecated Use new parameter approach instead see issue 48
     *             (http://code.google.com/p/gwt-fx/issues/detail?id=48)
     */
    public void setEffectLength(double new_EffectLengthSeconds) {
      effectLengthSeconds = new_EffectLengthSeconds;
    }

  }

  // Indicates if start > end in play parameters (different to inverse)
  boolean backwards = false;

  private Vector<NEffect> chainedEffects;

  double currentEffectPosition = 0.0;

  protected final double DEFAULT_EFFECT_LENGTH = 2.0;

  protected Element effectElement;

  protected boolean effectFinished = false;

  /**
   * Object to hold any effect handlers that are registered against this effect.
   * 
   * @deprecated
   */
  protected NEffectHandlerCollection effectHandlers;

  protected double effectLengthSeconds = DEFAULT_EFFECT_LENGTH;

  protected boolean interruptable = true;

  boolean inverted = false;

  // Store the real effect end point - usually this will be at position 1.0
  // unless overridden
  // by the user (see method interpolate for use of this variable).
  double requestedEffectEnd = 1.0;

  // Store the real effect start point - usually this will be at position 0.0
  // unless overridden
  // by the user (see method interpolate for use of this variable).
  double requestedEffectStart = 0.0;

  protected NEffectPanel thePanel;

  /**
   * Holds the Transition for the effect.
   */
  TransitionPhysics transition;

  /**
   * Add an EffectHandler to the effect.
   * 
   * @deprecated Use new event handling system.....
   */
  public void addEffectHandler(NEffectHandler theEffectHandler) {
    if (effectHandlers == null)
      effectHandlers = new NEffectHandlerCollection();
    effectHandlers.add(theEffectHandler);
  }

  /**
   * Override the Animate cancel to check that effect can be interrupted.
   */
  @Override
  public void cancel() {
    if (interruptable)
      super.cancel();
  }

  public void chain(final NEffect e) {
    if (chainedEffects == null)
      chainedEffects = new Vector<NEffect>();
    chainedEffects.add(e);
  }

  /**
   * Return the effect length in seconds.
   * 
   * @return Effect length in seconds.
   */
  public double getDuration() {
    return effectLengthSeconds;
  }

  /**
   * Return the current progress of the effect - a value somewhere between 0 and
   * 1.
   * 
   * @return
   */
  public double getProgress() {
    return currentEffectPosition;
  }

  public double getProgressInterpolated() {
    return interpolateWithoutUpdate(currentEffectPosition);
  }

  /**
   * Return the current Transition Physics Model used by the Effect.
   * 
   * @return
   */
  public TransitionPhysics getTransitionPhysics() {
    return transition;
  }

  /**
   * Initialises an effect on an effect panel.
   * 
   * @param thePanel The EffectPanel on which effect is being applied.
   */
  public void init(NEffectPanel thePanel) {
    this.thePanel = thePanel;
    if (transition == null)
      transition = new EaseInOutTransitionPhysics();
    if (chainedEffects != null) {
      for (Iterator<NEffect> it = chainedEffects.iterator(); it.hasNext();) {
        final NEffect theNext = (NEffect) it.next();
        theNext.init(thePanel);
        // this.addEffectHandler(new NEffectHandlerAdapter() {
        // @Override
        // public void postEvent(NEffect theEffect) {
        // theNext.play();
        // }
        // });
        this.addEffectCompletedHandler(new EffectCompletedHandler() {
          public void onEffectCompleted(EffectCompletedEvent event) {
            theNext.play();
          }
        });
      }
    }
    setUpEffect();
  }

  /**
   * Overridden version of the Animate class to allow us to swap in and out the
   * transition method. In GWT 1.5 this is a fixed calculation in the Animate
   * class.
   * 
   * We also use this method to fire any change events since it is not possible
   * elsewhere in the Animate framework, and in theory interpolate would be
   * called as many times as the update method.
   */
  @Override
  protected double interpolate(double progress) {
    double newVal = interpolateWithoutUpdate(progress);
    // Right, now we've done the maths, time to fire any change listeners that
    // may be registered
    // and finally, return the value.
    if (this.effectHandlers != null)
      effectHandlers.updatedEffect(this);
    // 1.6 event handling
    fireEvent(new EffectSteppingEvent(newVal));
    return newVal;
  }

  protected double interpolateWithoutUpdate(double progress) {
    // Check if effect is set up yet
    if (transition == null)
      return 1.0;
    // Check if the effect is backwards (e.g. requested start is higher than
    // requested end)
    if (!backwards) {
      // It is not, so just find the real progress
      progress = progress + requestedEffectStart;
      // Check that we are not now past the requested end of effect
      if (progress > requestedEffectEnd)
        progress = requestedEffectEnd;
    } else {
      // Effect is backwards, so just rework the progress calculation
      progress = requestedEffectStart - progress;
      if (progress < requestedEffectEnd)
        progress = requestedEffectEnd;
    }
    if ((progress == 1.0)) {
      if (inverted)
        return 0.0;
      else
        return 1.0;
    }
    if (progress == 0.0) {
      if (inverted)
        return 1.0;
      else
        return 0.0;
    }
    // Now get the position in animation based on the particular Transition to
    // be applied to this effect.
    double val = transition.getAnimationPosition(progress);
    // Finally, check if the effect is inverted, if so, deal with that.
    if (inverted)
      val = 1 - val;
    return val;
  }

  /**
   * 
   * Inverts an effect.
   * 
   */
  public void invert() {
    inverted = !inverted;
  }

  /**
   * Returns a boolean value indicating if the effect has finished or not.
   * 
   * @return True if the effect is finished; False otherwise
   */
  public boolean isFinished() {
    return effectFinished;
  }

  /**
   * Returns a boolean value to indicate if the effect is interruptable or not.
   * 
   * @return True if the effect is interruptable; false otherwise.
   */
  public boolean isInterruptable() {
    return interruptable;
  }

  /**
   * Returns a boolean value to indicate if the effect is inverted.
   * 
   * @return
   */

  public boolean isInverted() {
    return inverted;
  }

  /**
   * Called immediately after the animation is canceled. The default
   * implementation of this method calls {@link #onComplete()} only if the
   * animation has actually started running.
   * 
   * This overrides the version in the Animate class to allow the inclusion of
   * the effectHandlers.
   * 
   */
  @Override
  protected void onCancel() {
    super.onCancel();
    if (effectHandlers != null)
      effectHandlers.interruptedEffect(this);
    // New 1.6 event firing mechanism
    fireEvent(new EffectInterruptedEvent());
  }

  /**
   * Called immediately after the animation completes.
   * 
   * This overrides the version in the Animate class to allow the inclusion of
   * the effectHandlers.
   */
  @Override
  protected void onComplete() {
    super.onComplete();
    effectFinished = true;
    if (effectHandlers != null)
      effectHandlers.finishedEffect(this);
    // New 1.6 event firing mechanism
    fireEvent(new EffectCompletedEvent());
  }

  /**
   * Called immediately before the animation starts.
   * 
   * This overrides the version in the Animate class to allow the inclusion of
   * the effectHandlers.
   * 
   */
  @Override
  protected void onStart() {
    super.onStart();
    effectFinished = false;
    if (effectHandlers != null)
      effectHandlers.startedEffect(this);
    fireEvent(new EffectStartingEvent());
  }

  /**
   * Called when the animation should be updated.
   * 
   * The value of progress is between 0.0 and 1.0 inclusively (unless you
   * override the {@link #interpolate(double)} method to provide a wider range
   * of values). You can override {@link #onStart()} and {@link #onComplete()}
   * to perform setup and tear down procedures.
   * 
   * Note that onUpdate events are now triggered by a call to the interpolate
   * method whereas before they were through the direct update methods - this
   * needs to be done since the update method in GWT Animation is private.
   * 
   */
  @Override
  protected void onUpdate(double progress) {
    currentEffectPosition = progress;
    if (thePanel == null)
      return;
  }

  /**
   * The play method - defaults to playing effect from position 0.0 to 1.0.
   */
  public void play() {
    run((int) (effectLengthSeconds * 1000));
  }

  /**
   * The play method - allows specific definition of start and end positions.
   * 
   * Note that the portion of effect selected will run at the same tempo as the
   * whole effect, but the firing of any postEffect handlers will not occur
   * until the whole duration is complete.
   * 
   * For example, an effect with duration 2 seconds, but calling play(0.0, 0.5)
   * will appear to viewer to complete in 1 second, but will not fire any
   * postEvents until after 2 seconds.
   * 
   * @param start The requested start position.
   * @param end The requested end position.
   */
  public void play(double start, double end) {
    // If the start point is greater than the end point, then we need to play
    // the effect in reverse
    // But, note that this is not the same as inverting the effect.
    if (start > end)
      backwards = true;
    else
      backwards = false;
    // Store the start and end positions.
    requestedEffectStart = start;
    requestedEffectEnd = end;
    // Play the effect as normal
    play();
  }

  public void remove() {
    cancel();
    tearDownEffect();
  }

  /**
   * 
   * @deprecated
   */
  public void removeEffectHandler(NEffectHandler theEffectHandler) {
    if (effectHandlers != null)
      effectHandlers.remove(theEffectHandler);
  }

  /** Reset the effect */
  public void reset() {
    if (thePanel != null)
      onUpdate(0.0);
  }

  public void resumeBackwards() {
    this.play(getProgress(), 0.0);
  }

  public void resumeForwards() {
    this.play(getProgress(), 1.0);
  }

  public void resumeBackwards(double end) {
    this.play(getProgress(), end);
  }

  public void resumeForwards(double end) {
    this.play(getProgress(), end);
  }

  /**
   * Set the effect length.
   * 
   * @param new_EffectLengthSeconds
   */
  public void setDuration(double new_EffectLengthSeconds) {
    effectLengthSeconds = new_EffectLengthSeconds;
  }

  /**
   * @deprecated Use setDuration instead
   * @param individualEffectLength
   */
  public void setEffectLength(double individualEffectLength) {
    setDuration(individualEffectLength);
  }

  /**
   * Allow external user to specify position within an effect
   * 
   * @param progress the position to be set.
   */
  abstract public void setPosition(double progress);

  /*
   * 
   */
  public void setTransitionType(TransitionPhysics newTransition) {
    this.transition = newTransition;
  }

  /**
   * Method that is called to set the effect up. It is abstract here since we
   * are not defining an actual effect; but it is expected to be implemented by
   * implementations. (For example a Show effect may wish to have this method
   * hide the component before starting).
   */
  protected abstract void setUpEffect();

  /**
   * Method that is called when the effect is removed from a panel. Usually this
   * will not do anything, but for sure the Move effect needs to fix the style
   * properties that it sets up.
   */
  public abstract void tearDownEffect();

  /**
   * Ensures the existence of the handler manager.
   * 
   * @return the handler manager
   * */
  HandlerManager ensureHandlers() {
    return handlerManager == null ? handlerManager = new HandlerManager(this)
        : handlerManager;
  }

  /**
   * Fires an event. Usually used when passing an event from one source to
   * another.
   * 
   * @param event the event
   */
  public void fireEvent(GwtEvent<?> event) {
    if (handlerManager != null) {
      handlerManager.fireEvent(event);
    }
  }

  /**
   * Adds this handler to the widget.
   * 
   * @param <H> the type of handler to add
   * @param type the event type
   * @param handler the handler
   * @return {@link HandlerRegistration} used to remove the handler
   */
  protected final <H extends EventHandler> HandlerRegistration addHandler(
      final H handler, GwtEvent.Type<H> type) {
    return ensureHandlers().addHandler(type, handler);
  }

  public HandlerRegistration addEffectCompletedHandler(
      EffectCompletedHandler handler) {
    return addHandler(handler, EffectCompletedEvent.getType());
  }

  public HandlerRegistration addEffectStartingHandler(
      EffectStartingHandler handler) {
    return addHandler(handler, EffectStartingEvent.getType());
  }

  public HandlerRegistration addEffectSteppingHandler(
      EffectSteppingHandler handler) {
    return addHandler(handler, EffectSteppingEvent.getType());
  }

  public HandlerRegistration addEffectInterruptedHandler(
      EffectInterruptedHandler handler) {
    return addHandler(handler, EffectInterruptedEvent.getType());
  }

  HandlerManager handlerManager = new HandlerManager(DEFAULT_EFFECT_LENGTH);

  public HandlerManager getHandlers() {
    return handlerManager;
  }

  protected NEffectPanel applyEffect(String w, NEffect e) {
    String original = DOM.getInnerHTML(RootPanel.get(w).getElement());
    HTMLPanel newHTML = new HTMLPanel(original);
    DOM.setInnerHTML(RootPanel.get(w).getElement(), "");
    // String oldStyle =
    // StyleImplementation.getComputedStyles(RootPanel.get(w).getElement());
    // String oldStyle = DOM.getElementProperty(RootPanel.get(w).getElement(),
    // "style");
    String oldStyle = StyleImplementation.getComputedStyle(
        RootPanel.get(w).getElement(), "backgroundColor");
    DOM.setStyleAttribute(RootPanel.get(w).getElement(), "backgroundColor",
        "transparent");
    // DOM.setElementProperty(RootPanel.get(w).getElement(), "style", "");
    NEffectPanel ep = new NEffectPanel();
    ep.add(newHTML);
    ep.addEffect(e);
    RootPanel.get(w).add(ep);
    DOM.setStyleAttribute(ep.getWidget().getElement(), "backgroundColor",
        oldStyle);
    return ep;
  }
  
  private class TempWidget extends Widget{
    public TempWidget(Element e){
      setElement(e);
    }
  }

  protected NEffectPanel applyEffect(Widget w, NEffect e) {
    assert (w.isAttached());
    Panel src = (Panel) w.getParent();
    NEffectPanel eP;
    if (src instanceof NEffectPanel)
      eP = (NEffectPanel) src;
    else {
      eP = new NEffectPanel();
      if (src == null) {
        RootPanel p = ((RootPanel)w);
        com.google.gwt.user.client.Element el = p.getElement();
        w = new TempWidget(el);  
        RootPanel.get().add(eP);
        p.remove(w);
      } else {
        if (src instanceof SimplePanel) {
          ((SimplePanel) src).remove(w);
          ((SimplePanel) src).add(eP);
        } else if (src instanceof VerticalPanel) {
          int currIndex = ((VerticalPanel) src).getWidgetIndex(w);
          ((VerticalPanel) src).insert(eP, currIndex);
        } else if (src instanceof HorizontalPanel) {
          int currIndex = ((HorizontalPanel) src).getWidgetIndex(w);
          ((HorizontalPanel) src).insert(eP, currIndex);
        } else if (src instanceof DockPanel) {
          DockLayoutConstant currIndex = ((DockPanel) src).getWidgetDirection(w);
          ((DockPanel) src).add(eP, currIndex);
        } else if (src instanceof HTMLTable) {
          int col = ((HTMLTable) src).getCellCount(0);
          // Find the widget index
          Iterator<Widget> i = ((HTMLTable) src).iterator();
          boolean found = false;
          int count = 0;
          while (i.hasNext() && !found) {
            Widget compare = i.next();
            if (compare.equals(w))
              found = true;
            else
              count++;
          }
          // Turn index into x y co-ordinates
        }
      }
    }
    eP.add(w);
    eP.addEffect(e);
    return eP;
  }

}
